/* ========================================================================
C*
      PROGRAM cpf2irv
CC
CC NAME       :  cpf2irv
CC
CC PURPOSE    :  Convert CPF forecasrt ephemeris to irv format
CC               SAMPLE PROGRAM
CC
CC SR CALLED  :  cpf_inter_irv, jd2date, cephinitu
CC
CC
CC INCLUDES   :  ephi.h, jdcnvs.h                 
CC
CC AUTHOR     :  S. RIEPL, W. GURTNER
CC
CC
CC CREATED    :  10-FEB-2006            LAST MODIFIED :
CC
CC CHANGES    :  
CC
CC COPYRIGHT  :  BUNDESAMT FUER KARTOFRAPHIE UND GEODAESIE
CC      2005     GERMANY
CC                    
CC  MODIFIED:
CC
C*
=========================================================================== */
#include <stdio.h>
#include <math.h>
#include "../include/ephi.h"
#include "jdcnvs.h"
struct eh ephihdr;
struct ei ephi;
int debug= 0;	/* cephinitu debug flag */

int cephinitu(FILE *);

void cpf_inter_irv(double tt,int iunits,double *ttab,double *objtab_x,
	double *objtab_y,double *objtab_z,int ntab,int nintm,
	double *x,double *y,double *z,double *vx,double *vy,double *vz,
	int *ircode);

FILE   *lfncpf, *lfneph;

int main ()
{
      char cpffil[40],fileph[40];
      double x,y,z,vx,vy,vz;
      double tstart, tend, dt, tbias;
      double pi, tim, tim_0;
      int    i, iunits, irc, ierr;
      int nsteps;
      /* Number of values to be used for interpolation (sr hermite) */
      int    nval= 10;
      int year, month, day, hour, minute, second;
      double sumoth,sumpos,sumvel;
      int xp,yp,drt;
      
      pi= 4.e0*atan(1.e0);
      xp=yp=drt=0;
      
/* Radians */

/*
* Get Input Options from Input Queue
* ----------------------------------
*/
/* Filename of CPF prediction input file */
      printf ("CPF prediction file? ");
      scanf ("%s", cpffil);
/* Filename of ephemeris output file */
      printf ("Output file? ");
      scanf ("%s", fileph);
/* Start time (Modified Julian Date) */
      printf ("Start time (Modified Julian Date)? ");
      scanf ("%lf", &tstart);
/* End time (MJD) */
      printf ("Stop  time (Modified Julian Date)? ");
      scanf ("%lf", &tend);
/* Increment (seconds) */
      printf ("Time increment (seconds)? ");
      scanf ("%lf", &dt);
/* Time Bias (seconds) */
      printf ("A priori time bias (sec, will be added to epochs)? ");
      scanf ("%lf", &tbias);
/*
C  Read CPF File
C  -------------
C
C  JDI,JDF,JDS,OUTV,NV USED BELOW ARE STORED BY SUBR EPHINITU INTO COMMON
C  (EPHEMERIDES: EPOCH, X,Y,Z, NUMBER OF VECTORS)
C  (SEE INCLUDE FILES)
*/
      if ((lfncpf=fopen(cpffil,"r"))==NULL)  {
        printf ("*** PGM CPF_PGM: Error opening CPF file %s\n",cpffil);
        exit(1); }

      ierr= cephinitu (lfncpf);
      fclose(lfncpf);
      if (ierr != 0) {
        printf("*** PGM CPFINT: Error reading CPF file\n");
        exit(1);
      }

/*
 * Is this an slr target or llr/tlr? */
      if (ephihdr.ltcorr == 1 || ephihdr.tar_type == 2 || ephihdr.compat != 1)
        {
          printf("PGM CPF_PGM: File %s\n", cpffil);
          printf("             requires LLR/TLR prediction software!");
          exit(1);
        }

/* Time units (for jds): seconds */
      iunits=0;

/* File to store the interpolated values */
    if ((lfneph=fopen(fileph,"w"))==NULL)  {
        printf ("Could not open file %s\n",fileph);
        exit(1); }

/* Test modified Julian date */
      nsteps=(tend-tstart)*86400.0e0/dt;
      /* fprintf(lfneph,"Epoch=%15.9f\n",tstart); */
      tim_0=(tstart-(ephi.jdi[0]+ephi.jdf[0]-2400000.5e0))*86400.0e0;
      for(i=0;i<nsteps;i++)	{
      	  tim=tim_0+i*dt;
/*        Relative time to start of cpf file 
 *        JDI (FULL DAYS), JDI (FRACTION) GIVEN IN JULIAN DATE
 *        JDS: DAYS RELATIVE TO FIRST EPOCH */

          cpf_inter_irv(tim+tbias,iunits,ephi.jds,&ephi.outv[0][0],&ephi.outv[1][0],
                   &ephi.outv[2][0],ephi.nv,nval,&x,&y,&z,&vx,&vy,&vz,&irc);
          jd2date(ephi.jdi[0]+ephi.jdf[0]+tim/86400.0e0,&year,&month,&day,&hour,&minute,&second);
          sumoth=year+month+day+hour+minute+second+ephihdr.sic+0+ephihdr.eph_seq+xp+yp+drt;
          sumpos=x+y+z; sumvel=vx+vy+vz;
          fprintf(lfneph,"%s\n%5d%3d%3d%3d%3d%5.1f   %18.6f%18.6f%18.6f\n %4d%6d%6d     %18.6f%18.6f%18.6f\n %6d %6d %6d %18.1f%18.6f%18.6f\n",
              "WTZ PURE IRV",year,month,day,hour,minute,(double)(second),
              x,y,z,ephihdr.sic,0,ephihdr.eph_seq,vx,vy,vz,xp,yp,drt,
              sumoth,sumpos,sumvel);
        }
        
      fclose(lfneph);
  return 0;
}
