#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "../include/ephi.h"

struct eh ephihdr;
struct ei ephi;

int debug= 0; /* for ephinterp.c */

/* cpf_convert
 * convert automatically between versions 1 and 2 of the ILRS cfp format
 *
 * Run:
 *   cpf_conv <cpf_v1>
 *      -> outputs file name and file <cpf_v2> in a name derived from CPF
 *   cpf_conv <cpf_v2>
 *      -> outputs file name and file <cpf_v1> in a name derived from CPF
 *   cpf_conv <cpf> [dir_name]
 *      -> outputs file into directory dir_name rather than in the current
 *         directory
 *
 * Compile: gcc -o cpf_conv cpf_conv.c
 *
 * rlr - 03/06/18
 *
 * History:
 *  05/23/18 - Added direcory argument. rlr.
 *  05/25/18 - Don't assume "H1" is first record in file (could be "00"). rlr.
 */
int
main (argc, argv)
     int argc;
     char *argv[];
{
  char str[256];
  char file_out[256];
  char ephstr[256];
  FILE *str_in, *str_out;

  if ((str_in = fopen (argv[1], "r")) == NULL)
    {
      printf ("Could not open file %s\n", argv[1]);
      exit (1);
    }

  // pre-pend a directory name if required.
  if (argc == 3)
    {
      strcpy (file_out, argv[2]);
      strcat (file_out, "/");
      strcat (file_out, argv[1]);
    }
  else
    strcpy (file_out, argv[1]);

  // Read until "H1" is found. Use it to create new file name and open.
  while (fgets (ephstr, 256, str_in) != NULL)
    {
      if (strncmp (ephstr, "h1", 2) == 0 || strncmp (ephstr, "H1", 2) == 0)
	{
          sscanf (ephstr, "%*s %*s %d", &ephihdr.version);
         
          if (ephihdr.version == 1)
            {
	      sscanf (&ephstr[7], "%2d  %3c %4d %2d %2d %2d  %4d %s %s",
		      &ephihdr.version, ephihdr.eph_source, &ephihdr.epyear,
		      &ephihdr.epmon, &ephihdr.epday, &ephihdr.ephour,
		      &ephihdr.eph_seq, ephihdr.tar_name, ephihdr.eph_notes);

              // convert to v2 eph_seq_doy and eph_seq_subday
              ephihdr.eph_seq_doy= ephihdr.eph_seq/10- 500;
              ephihdr.eph_seq_subday= ephihdr.eph_seq % 10;

              // Figure out file name and write H1.
              sprintf (str, "%03d%02d.%c%c%c", 
                       ephihdr.eph_seq_doy, ephihdr.eph_seq_subday, 
                       tolower(ephihdr.eph_source[0]),
                       tolower(ephihdr.eph_source[1]),
                       tolower(ephihdr.eph_source[2]));
              strcpy (strstr (file_out, ".")-4, str);

              if ((str_out = fopen (file_out, "w")) == NULL)
                {
                  printf ("Could not open file %s\n", file_out);
                  exit (1);
                }
              printf ("CPF V2 file_out: [%s]\n", file_out);
            }
          else if (ephihdr.version == 2)
            {
	      sscanf (&ephstr[3], "%*s %d %s %d %d %d %d %d %d %s %s",
		      &ephihdr.version, ephihdr.eph_source, &ephihdr.epyear,
		      &ephihdr.epmon, &ephihdr.epday, &ephihdr.ephour,
		      &ephihdr.eph_seq_doy, &ephihdr.eph_seq_subday, 
                      ephihdr.tar_name, ephihdr.eph_notes);

              // convert to v1 eph_seq
              ephihdr.eph_seq= (ephihdr.eph_seq_doy+500)* 10;

              // just take the least significant digit
              ephihdr.eph_seq+= ephihdr.eph_seq_subday % 10;
              sprintf (str, "%04d.%c%c%c", 
                       ephihdr.eph_seq, 
                       tolower(ephihdr.eph_source[0]),
                       tolower(ephihdr.eph_source[1]),
                       tolower(ephihdr.eph_source[2]));

              // Figure out file name and write H1.
              strcpy (strstr (file_out, ".")-5, str);
              if ((str_out = fopen (file_out, "w")) == NULL)
                {
                  printf ("Could not open file %s\n", argv[2]);
                  exit (1);
                }
              printf ("CPF V1 file_out: [%s]\n", file_out);
            }
	}
    }
  rewind (str_in);

  // Now, process the entire file.
  while (fgets (ephstr, 256, str_in) != NULL)
    {
      if (strncmp (ephstr, "h1", 2) == 0 || strncmp (ephstr, "H1", 2) == 0)
        {
          // fields are in memory already due to search for "H1" above.
          if (ephihdr.version == 1)
            {
	      fprintf (str_out, "H1 CPF %d %s %d %d %d %d %d %d %-s %-s\n",
		      2, ephihdr.eph_source, ephihdr.epyear,
		      ephihdr.epmon, ephihdr.epday, ephihdr.ephour,
		      ephihdr.eph_seq_doy, ephihdr.eph_seq_subday, 
                      ephihdr.tar_name, ephihdr.eph_notes);
            }
          else if (ephihdr.version == 2)
            {
	      fprintf (str_out, 
                      "H1 CPF %2d  %-3s %4d %2d %2d %2d  %4d %-10s %-10s\n",
		      1, ephihdr.eph_source, ephihdr.epyear,
		      ephihdr.epmon, ephihdr.epday, ephihdr.ephour,
		      ephihdr.eph_seq, ephihdr.tar_name, ephihdr.eph_notes);
            }
	}
      else if (strncmp (ephstr, "h2", 2) == 0
	       || strncmp (ephstr, "H2", 2) == 0)
	{
          if (ephihdr.version == 1)
            {
	      sscanf (&ephstr[3],
	 	      "%8d %4d %8d %4d %2d %2d %2d %2d %2d %4d %2d %2d %2d %2d %2d %5d %1d %1d %2d %1d %1d",
		      &ephihdr.cospar_id, &ephihdr.sic, &ephihdr.norad,
		      &ephihdr.sttpyear, &ephihdr.sttpmonth,
		      &ephihdr.sttpday, &ephihdr.sttphour, &ephihdr.sttpmin,
		      &ephihdr.sttpsec, &ephihdr.endpyear, &ephihdr.endpmonth,
		      &ephihdr.endpday, &ephihdr.endphour, &ephihdr.endpmin,
		      &ephihdr.endpsec, &ephihdr.ephsep, &ephihdr.compat,
		      &ephihdr.tar_type, &ephihdr.ref_frame, &ephihdr.rot_type,
		      &ephihdr.cofm_app);

               // Convert to v2 target class and location
               // incomplete: Could include LRO, ELT, etc.
               ephihdr.tar_class= ephihdr.tar_type;
               ephihdr.tar_loc= 1;
               if (ephihdr.tar_type == 2)       // old lunar reflector entry
                 {
                   ephihdr.tar_class= 1;
                   ephihdr.tar_loc= 3;
                 }
	      fprintf (str_out,
		      "H2 %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
		      ephihdr.cospar_id, ephihdr.sic, ephihdr.norad,
		      ephihdr.sttpyear, ephihdr.sttpmonth,
		      ephihdr.sttpday, ephihdr.sttphour, ephihdr.sttpmin,
		      ephihdr.sttpsec, ephihdr.endpyear, ephihdr.endpmonth,
		      ephihdr.endpday, ephihdr.endphour, ephihdr.endpmin,
		      ephihdr.endpsec, ephihdr.ephsep, ephihdr.compat,
		      ephihdr.tar_class, ephihdr.ref_frame, ephihdr.rot_type,
		      ephihdr.cofm_app, ephihdr.tar_loc);
            }
          else if (ephihdr.version == 2)
            {
	      sscanf (&ephstr[3],
		      "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
		      &ephihdr.cospar_id, &ephihdr.sic, &ephihdr.norad,
		      &ephihdr.sttpyear, &ephihdr.sttpmonth,
		      &ephihdr.sttpday, &ephihdr.sttphour, &ephihdr.sttpmin,
		      &ephihdr.sttpsec, &ephihdr.endpyear, &ephihdr.endpmonth,
		      &ephihdr.endpday, &ephihdr.endphour, &ephihdr.endpmin,
		      &ephihdr.endpsec, &ephihdr.ephsep, &ephihdr.compat,
		      &ephihdr.tar_class, &ephihdr.ref_frame, &ephihdr.rot_type,
		      &ephihdr.cofm_app, &ephihdr.tar_loc);

               // Convert to v1 target type
               ephihdr.tar_type= ephihdr.tar_class;
               if (ephihdr.tar_class == 1 && ephihdr.tar_loc == 3)
                 {
                   ephihdr.tar_type= 2;
                 }
	      fprintf (str_out,
	 	      "H2 %8d %4d %8d %4d %02d %02d %2d %2d %2d %4d %02d %02d %2d %2d %2d %5d %1d %1d %2d %1d %1d\n",
		      ephihdr.cospar_id, ephihdr.sic, ephihdr.norad,
		      ephihdr.sttpyear, ephihdr.sttpmonth,
		      ephihdr.sttpday, ephihdr.sttphour, ephihdr.sttpmin,
		      ephihdr.sttpsec, ephihdr.endpyear, ephihdr.endpmonth,
		      ephihdr.endpday, ephihdr.endphour, ephihdr.endpmin,
		      ephihdr.endpsec, ephihdr.ephsep, ephihdr.compat,
		      ephihdr.tar_type, ephihdr.ref_frame, ephihdr.rot_type,
		      ephihdr.cofm_app);
            }
	}
      else if (strncmp (ephstr, "h3", 2) == 0
	       || strncmp (ephstr, "H3", 2) == 0)
	{
	  sscanf (&ephstr[3],
            "%d %d %d %d %d %d %d %d %d",
		  &ephihdr.atrk_ro_0, &ephihdr.xtrk_ro_0, &ephihdr.rtrk_ro_0,
		  &ephihdr.atrk_ro_1, &ephihdr.xtrk_ro_1, &ephihdr.rtrk_ro_1,
		  &ephihdr.atrk_ro_2, &ephihdr.xtrk_ro_2, &ephihdr.rtrk_ro_2);
          if (ephihdr.version == 1)
            {
              fprintf (str_out, "H3 %d %d %d %d %d %d %d %d %d\n",
		  ephihdr.atrk_ro_0, ephihdr.xtrk_ro_0, ephihdr.rtrk_ro_0,
		  ephihdr.atrk_ro_1, ephihdr.xtrk_ro_1, ephihdr.rtrk_ro_1,
		  ephihdr.atrk_ro_2, ephihdr.xtrk_ro_2, ephihdr.rtrk_ro_2);
            }
          else if (ephihdr.version == 2)
            {
              fprintf (str_out, "H3 %5d %5d %5d %5d %5d %5d %5d %5d %5d\n",
		  ephihdr.atrk_ro_0, ephihdr.xtrk_ro_0, ephihdr.rtrk_ro_0,
		  ephihdr.atrk_ro_1, ephihdr.xtrk_ro_1, ephihdr.rtrk_ro_1,
		  ephihdr.atrk_ro_2, ephihdr.xtrk_ro_2, ephihdr.rtrk_ro_2);
            }
	}
      else if (strncmp (ephstr, "h4", 2) == 0
	       || strncmp (ephstr, "H4", 2) == 0)
	{
          if (ephihdr.version == 1)
            {
	      sscanf (&ephstr[3],
		    "%lf %lf %lf %lf",
		    &ephihdr.prf, &ephihdr.xpond_xmit_delay,
		    &ephihdr.xpond_utc_off, &ephihdr.xpond_osc_drift);
              ephihdr.xpond_clock_ref_time= 0.e0;
              fprintf (str_out, 
		    "H4 %.5lf %.4lf %.2lf %.2lf %.12lf\n",
		    ephihdr.prf, ephihdr.xpond_xmit_delay,
		    ephihdr.xpond_utc_off, ephihdr.xpond_osc_drift,
                    ephihdr.xpond_clock_ref_time);
            }
          else if (ephihdr.version == 2)
            {
	      sscanf (&ephstr[3],
		    "%lf %lf %lf %lf %lf",
		    &ephihdr.prf, &ephihdr.xpond_xmit_delay,
		    &ephihdr.xpond_utc_off, &ephihdr.xpond_osc_drift,
                    &ephihdr.xpond_clock_ref_time);
              fprintf (str_out,
		    "H4 %12.5lf %10.4lf %11.2lf %11.2lf\n",
		    ephihdr.prf, ephihdr.xpond_xmit_delay,
		    ephihdr.xpond_utc_off, ephihdr.xpond_osc_drift);
            }
	}
      else if (strncmp (ephstr, "h5", 2) == 0
	       || strncmp (ephstr, "H5", 2) == 0)
	{
	  sscanf (&ephstr[3], "%lf", &ephihdr.cofm_corr);
          if (ephihdr.version == 1)
            {
              fprintf (str_out, "H5 %.4lf\n", ephihdr.cofm_corr);
            }
          else if (ephihdr.version == 2)
            {
              fprintf (str_out, "H5 %7.4f\n", ephihdr.cofm_corr);
            }
	}
      else
	{
	  fputs (ephstr, str_out);
	}
    }
  fclose (str_in);
  fclose (str_out);
}
