!        PROGRAM cpfpred (INPUT,OUTPUT,TTY,UTPOLE,POLDAT,MITEPHB, 
!     X TAPE5=INPUT,TAPE6=OUTPUT,TAPE10=TTY) 
      IMPLICIT DOUBLE PRECISION (a-h,o-z) 
C
C  Compute predictions from a cpf file.

C  This is test driver to demonstrate the performance of the ilrs cpf
C  prediction sample code. This software is compatible and rigorous 
C  (to a point) for cpfs for all objects including artificial earth 
C  satellites, reflectors on the moon, and itransponders.
C
C  Program usage: cpfpred -g geodname -i ephname [-o outname] [-s stepsec]
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C-----------------------------------------------------------------------
!........... INCLUDE FILES .....................................  
      INCLUDE  'bounce.inc'
      INCLUDE  'dirctn.inc'
      INCLUDE  'intrng.inc'
      INCLUDE  'obsinf.inc' 
      INCLUDE  'feature.inc'
  
      COMMON /twopi/ twopi,cdr,casr,ctsr
  
      INCLUDE  '../include/ephihdr.inc'

      INCLUDE  '../include/ephi.inc'

      COMMON /deb/ debug
      LOGICAL debug

      DOUBLE PRECISION lhatrn(6),azaltrn(6),lharcv(6),azalrcv(6), 
     & lratrn, lrarcv 
      DOUBLE PRECISION stepsec
      DOUBLE PRECISION vlight
      DOUBLE PRECISION ged2gec 
      INTEGER d1(5),d2(5),idf(6),idl(6) 
      INTEGER itrimlen, lenout
      LOGICAL coloc, outopen, backang
      LOGICAL risen 
  
      CHARACTER*256 ephname,geodname,outname, outname_full
      CHARACTER*4 sname

      DATA TWOPI/6.283185307179586D0/
      DATA CDR /1.74532925199433D-02/
      DATA CASR/4.84813681109536D-06/
      DATA CTSR/7.27220521664304D-05/

!  MOON HAS NOT YET RISEN:
      DATA risen /.FALSE./
      DATA stepsec/900.d0/
      DATA vlight/0.299792458d+06/
      DATA outopen /.FALSE./
CC      DATA backang /.FALSE./
      DATA backang /.TRUE./
      DATA debug /.FALSE./
CC      DATA debug /.TRUE./

C Reflector 3
      DATA feature /1735476.394000d0,3.61132590d0,26.15524990d0/
!-----------------------------------------------------------------
  
      nargs=ipargs(geodname,ephname,outname,stepsec)
  
      CALL ephinit (ephname,ierr)
      IF (ierr .NE. 0) STOP

      CALL cnvcoor(1.d0,feature,bodftr)
  
      minelv=0.d0
  
      CALL read_geod (geodname, sname, trad,tlong,tgdlat,tgclat,
     &    rrad,rlong,rgdlat,rgclat, coloc)
      IF (.NOT. coloc) backang= .TRUE.

! Load observatory coordinates
      transm(1)=trad
      transm(2)=tlong
      transm(3)=tgclat
      receiv(1)=rrad
      receiv(2)=rlong
      receiv(3)=rgclat
  
!  Insure time starts on even second.
      tjdint=jdi(1)
!     jdf is already in utc days
      sutct=jdf(1)*86400.d0 + 5*ephsep
CC      sutct=jdf(1)*86400.d0 + 5*ephsep- 0.0059136d0 ! TEST FOR MARS
      if (debug) WRITE (*,9999) jdi(1),jdf(1),sutct
 9999	format("jd= ",f10.1,f20.12,f20.8)
      nrcd=1
  
!  Time loop
!  DO WHILE time < endtime
   20 fjdutc=sutct/86400.d0
      tjdutc=tjdint+fjdutc
      CALL jdtgr (tjdint, fjdutc, d1(3), d1(1), d1(2), d1(4), d1(5),
     &secs1)

C     For round-off...
      IF (secs1.ge.59.999995d0) THEN
        CALL jdtgr (tjdint, fjdutc+1.d-13, d1(3), d1(1), d1(2), d1(4),
     &   d1(5), secs1)
      END IF
      d1(3)=d1(3)+1900

      if (debug) WRITE (*,*) "==>date:",d1,secs1
  
!:Compute range and point angles
CC	write(*,*) "ephsep:",ephsep
      CALL predict (tjdint, sutct, vlight, backang,
     &ttltim, lhatrn, azaltrn, lratrn, lharcv, azalrcv, lrarcv)
      if (debug) WRITE (*,*) "gpst",ttltim,lhatrn,azaltrn,lharcv,azalrcv
  
!  Check for moon risen or set before writing anything
      IF (azaltrn(3).lt.minelv.and.risen) GO TO 55
      IF (azaltrn(3).lt.minelv) GO TO 50
      risen=.true.
      if (.not.outopen) then
        outopen= .TRUE.
        lenout= itrimlen (outname)
C	The listing file 
        WRITE (outname_full,100) outname(1:lenout),
     &    mod(d1(3),100), d1(1), d1(2), d1(4), d1(5)
  100 FORMAT (A,"_y",i2.2, i2.2, i2.2, "t", i2.2, i2.2) 
        OPEN (3,file=outname_full)
        if (debug) write (*,*) "Writing prediction file: ",
     &    outname_full(1:itrimlen(outname_full))

C        write greeting
         WRITE (3,10)
   10    FORMAT (18x,'ILRS Laser Predictions Sample Software'/18x,
     &     38('-'))
  
      endif
  
!  Print time, reflector number, point angles, & range to output file
      WRITE (3,300) nrcd,tjdutc,d1,secs1
  300 FORMAT (/1x,'Record: ',i3,3x,'Time:',f15.6,3x,2(i2.2,'/'),i4.4,2x,
     &2(i2.2,':'),f10.7/3x,'SIC',3x,'Az',8x,'El',8x,'LHA',7x,'RA',8x,
     &'Dec',7x,'Range')
  
      IF (lratrn.lt.0.d0) lratrn=lratrn+24.d0
      IF (lratrn.ge.24.d0) lratrn=lratrn-24.d0

      if (debug) WRITE (*,40) sic,azaltrn(2),azaltrn(3),lhatrn(2),
     &lratrn,lhatrn(3), ttltim
      WRITE (3,40,err=70) sic,azaltrn(2),azaltrn(3),lhatrn(2),lratrn,
     &lhatrn(3),ttltim
   40 FORMAT (1x,i5,5f10.4,f18.11)
      IF (backang)
     &WRITE (3,40,err=70) sic,azalrcv(2),azalrcv(3),lharcv(2),lrarcv,
     &lharcv(3),ttltim

      CALL jdtgr (tjdint, fjdutc+1.d-9+ttltim/86400.d0, d1(3), d1(1), 
     &   d1(2), d1(4), d1(5), secs1)
      d1(3)=d1(3)+1900

      WRITE (3,301) nrcd,tjdutc+ttltim/86400.d0,d1,secs1
  301 FORMAT (/1x,'Receive:',i3,3x,f20.10,3x,2(i2.2,'/'),i4.4,2x,
     &2(i2.2,':'),f10.7);
      WRITE (3,302) ttltim*vlight/2.d0
  302 FORMAT ("Range in km: ",f20.9)
  
      nrcd=nrcd+1
  
!  Increase time by step size
   50 CONTINUE
  
      if (debug) write(*,*) "Step",tjdutc,jdi(nv),jdf(nv),
     &  jdi(nv)+jdf(nv)-5*ephsep/86400.d0

C  Done?
      IF (tjdutc.ge.jdi(nv)+jdf(nv)-(5*ephsep+stepsec)/86400.d0) 
     &  GO TO 60
  
C  Keep going...
      sutct=sutct+stepsec
      GO TO 20

!  Close this ephemeris and wait for a new one.
   55 CONTINUE
      CLOSE (3)
      IF (tjdutc.ge.jdi(nv)+jdf(nv)-5*ephsep/86400.d0) GO TO 60
      outopen= .FALSE.
      risen= .FALSE.
      GO TO 20
  
!  Complete output ephemeris processing
   60 CONTINUE
!C FIXME close (in_file)
      STOP

   70 WRITE (*,*) "write error"
      END
  
C-----------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION ged2gec (gdlat)
C  GED2GEC computes the geocentric latitude from the geodetic. This is an
C  approximation, although a pretty good one.
C
C  Input: gedlat: Geodetic latitude (dp degrees)
C  Output: <ged2gec>: Geocentric latitude (dp degrees)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C-----------------------------------------------------------------------
      DOUBLE PRECISION gdlat
      DOUBLE PRECISION if,f,f2,f3,gl
  
      COMMON /twopi/ twopi,cdr,casr,ctsr
      DOUBLE PRECISION twopi,cdr,casr,ctsr

      DATA if/298.257d+00/
  
      f=1.d0/if
      f2=f*f
      f3=f*f2
      gl=gdlat*cdr
      dlat=(f+f2/2.d0)*dsin(2.d0*gl)-((f2+f3)/2.0d0)*dsin(4.d0*gl)+f3/
     &3.d0*dsin(6.d0*gl)
      ged2gec=(gl-dlat)/cdr
!C        write (*,*) "geod, geoc, dlat", GDLAT,GED2GEC,dlat/CDR
      RETURN
      END

C-----------------------------------------------------------------------
      FUNCTION ipargs (geodname,ephname,outname,stepsec)
C
C Grab arguments from command line.
C  Note: The calls to iargc() and getarg() below are sysem dependent
C        and may need to be replaced...
C
C  Input: none
C  Output:
C	geodname: name of file containing station coordinates
C	ephname:  name of input cpf file
C	outname:  name of output prediction file.
C		  Default is "stdout".
C	stepsec:  Number of seconds between predictions.
C		  Default is ephsep/4.
C
C  Program usage: cpfpred -g geodname -i ephname [-o outname] [-s stepsec]
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C-----------------------------------------------------------------------

      CHARACTER*256 ephname,geodname,outname
      double precision stepsec
      CHARACTER anarg*256,msg*80
  
      outname= "STDOUT"  
      numargs=iargc()
      iprdargs=numargs
      narg=0
!===================================================================
   10 IF (narg.ge.numargs) GO TO 20
      narg=narg+1
      CALL getarg (narg, anarg)
      lena=itrimlen(anarg)
  
!C Prediction control file name is overridden...
      IF (anarg(1:2).eq.'-g') THEN
        narg=narg+1
        IF (narg.gt.numargs) THEN
          msg=' No file name follows "-g" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        CALL getarg (narg, anarg)
        lena=itrimlen(anarg)
        IF (anarg(1:1).eq.'-') THEN
          msg=' No file name follows "-g" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        IF (lena.gt.256) THEN
          msg=' Control file name too long.'//char(7)
          CALL error (2, msg, 2, nargs)
        END IF
        geodname= anarg
! + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
!C Prediction ephemeris file name is set...
      ELSE IF (anarg(1:2).eq.'-i') THEN
        narg=narg+1
        IF (narg.gt.numargs) THEN
          msg=' No file name follows "-i" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        CALL getarg (narg, anarg)
        lena=itrimlen(anarg)
        IF (anarg(1:1).eq.'-') THEN
          msg=' No file name follows "-i" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        IF (lena.gt.256) THEN
          msg=' ephemeris file name too long.'//char(7)
          CALL error (2, msg, 2, nargs)
        END IF
        ephname=anarg
! + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
!C Prediction ephemeris file name is set...
      ELSE IF (anarg(1:2).eq.'-o') THEN
        narg=narg+1
        IF (narg.gt.numargs) THEN
          msg=' No file name follows "-o" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        CALL getarg (narg, anarg)
        lena=itrimlen(anarg)
        IF (anarg(1:1).eq.'-') THEN
          msg=' No file name follows "-o" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        IF (lena.gt.256) THEN
          msg=' ephemeris file name too long.'//char(7)
          CALL error (2, msg, 2, nargs)
        END IF
        outname=anarg
! + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
!C Prediction ephemeris file name is set...
      ELSE IF (anarg(1:2).eq.'-s') THEN
        narg=narg+1
        IF (narg.gt.numargs) THEN
          msg=' No argument follows "-o" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        CALL getarg (narg, anarg)
        lena=itrimlen(anarg)
        IF (anarg(1:1).eq.'-') THEN
          msg=' No argument follows "-o" on command line.'
          CALL error (2, msg, narg, numargs)
        END IF
        IF (lena.gt.256) THEN
          msg=' argument too long.'//char(7)
          CALL error (2, msg, 2, nargs)
        END IF
        read (anarg,*) stepsec
! + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
      ELSE
        msg=' Bad command line argument: "'//anarg(1:lena)//'"'
        CALL error (2, msg, narg, lena)
      END IF
! + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
      GO TO 10
   20 CONTINUE

      END

C-----------------------------------------------------------------------
      subroutine read_geod (geodname, sname, trad,tlong,tgdlat,tgclat,
     &    rrad,rlong,rgdlat,rgclat, coloc)
C  READ_GEOD 
C  Read the station coordinates from a file.
C
C  Input: 
C	geodname: name of station coordinate file.
C	debug: want printed debug output? (logical; from /deb/.
C
C  Output:
C	sname: station name (char)
C	trad:  Laser transmitter radius (height above geocenter).
C	tlong: Laser transmitter longitude
C	tgdlat:Laser transmitter geodetic latitude
C	tgdcat:Laser transmitter geocentric latitude
C	rrad:  Laser receiver radius (height above geocenter).
C	rlong: Laser receiver longitude
C	rgdlat:Laser receiver geodetic latitude
C	rgdcat:Laser receiver geocentric latitude
C	coloc: Are transmitter and receiver the same? (logical)
C	   (All angles about are d.p. degrees.)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C-----------------------------------------------------------------------

      IMPLICIT DOUBLE PRECISION (a-h,o-z)

      COMMON /twopi/ twopi,cdr,casr,ctsr

      COMMON /deb/ debug
      LOGICAL debug

      character geodname*256, sname*4
      double precision trad,tlong,tgclat,tgdlat,
     &    rrad,rlong,rgclat,rgdlat, theight, rheight
      LOGICAL coloc
      double precision ged2gec
      double precision flat, ae
      DATA flat /298.255D0/, ae /6.378137D6/

      OPEN(1,file=geodname,ERR=990)
      READ(1,100,ERR=991) sname,tgdlat,tlong,theight,tgclat,
     &    rgdlat,rlong,rheight,rgclat
 100  FORMAT(1x,/,/,22X,A4,14X,8F13.7)
      IF (debug)
     &WRITE(*,100) sname,tgdlat,tlong,theight,tgclat,
     &    rgdlat,rlong,rheight,rgclat

      coloc = .FALSE.
      if ((tgdlat .NE. rgdlat .OR. tgclat .NE. rgclat .OR. 
     &    tlong .NE. rlong .OR. theight .NE. rheight) .OR.
     &     (rgdlat .NE. 0.d0 .AND. rgclat .NE. 0.d0 .AND. 
     &      rlong .NE. 0.d0 .AND. rheight .NE. 0.d0)) coloc= .TRUE.
      if (debug) write(*,*) "coloc= ",coloc

C  In case we want backleg angles anyway.
      if (coloc) then
        rgdlat= tgdlat
        rgclat= tgclat
        rlong= tlong
        rheight= theight
      endif

c Get the irritating geocentric latitude for time SDE calculations, etc.
      IF (tgclat.eq.0.d0.or.tgclat==tgdlat) tgclat=ged2gec(tgdlat)
      IF (rgclat.eq.0.d0.or.rgclat==rgdlat) rgclat=ged2gec(rgdlat)
      
c  Calculate geocentric rectangular coordinates of site
      ECC2 = 2.D0/FLAT  -  1.D0/(FLAT*FLAT)
      CLNG = dCOS(tLONG*cdr)
      SLNG = dSIN(tLONG*cdr)
      CLAT = dCOS(tGDLAT*cdr)
      SLAT = dSIN(tGDLAT*cdr)
      XN   = AE/DSQRT(1.D0-ECC2*SLAT**2)
      gdtrad= xn+ theight
      XSITE = gdtrad*CLAT*CLNG
      YSITE = gdtrad*CLAT*SLNG
      ZSITE = (gdtrad - ECC2*XN)*SLAT
      trad= dsqrt(xsite*xsite+ysite*ysite+zsite*zsite)
      CLNG = dCOS(rLONG*cdr)
      SLNG = dSIN(rLONG*cdr)
      CLAT = dCOS(rGDLAT*cdr)
      SLAT = dSIN(rGDLAT*cdr)
      XN   = AE/DSQRT(1.D0-ECC2*SLAT**2)
      gdrrad= xn+ rheight
      XSITE = gdrrad*CLAT*CLNG
      YSITE = gdrrad*CLAT*SLNG
      ZSITE = (gdrrad - ECC2*XN)*SLAT
      rrad= dsqrt(xsite*xsite+ysite*ysite+zsite*zsite)

      if (debug) write(*,*) "RAD DIFF= ", trad- 6374609.83000d0
      if (debug) write(*,999) trad,tlong,tgclat,tgdlat,
     &    rrad,rlong,rgclat,rgdlat, theight, rheight
 999	format(10f17.8)
      if (debug)
     &  write(*,*) "xn, th, rh, tr, rr",xn,theight,rheight,trad,rrad

      CLOSE (1)
      return

 990  WRITE (*,*) "Cannot open geodetics file:",geodname
      stop
 991  WRITE (*,*) "Read error from geodetics file:",geodname
      stop
      end

 
C-----------------------------------------------------------------------
      integer function itrimlen(array)
C
C  find the length of an array to the first blank (or
C  null, or end of array)
C
C  Input
C	array - character array
C
C  Output
C	<itrimlen> - integer length
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
C
      character array*(*), nul*1
 
      nul=char(0)
      length= len(array)
      itrimlen= length

      do 10 i=1,length
        if (array(i:i).NE.' ' .AND. array(i:i).NE.nul) goto 10
          itrimlen= i-1
          goto 90
10    continue

90    return
c
      end

C-----------------------------------------------------------------------
      subroutine error(icode,loc,ierr,ios)
C
C  Error code printing routine. Pretty primitive!
C
C  Input
C	icode	- source of error (Not used??)
C	loc	- Subroutine where error occurred.
C	ierr	- Error number 
C	ios	- FORTRAN error number
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      implicit double precision (a-h,o-z)

      character*(*) loc

      write(*,1000) loc,ierr,ios
1000      format(1X,A80,/,' Subroutine ERROR: #',I8,',  #',I8)
      stop

      end

