      SUBROUTINE predict (tjdint, sutct, vlight, backang, 
     &ttltim, lhatrn, azaltrn, ratrn, lharcv, azalrcv,rarcv)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
  
C  Calculate round-trip range and point angles between earth site and 
C  ranging target.
C
C  Input
C	tjdint	- d.p. whole part of Julian date
C	secutc  - d.p. seconds of fractional Julian day
C	vlight	- d.p. velocity of light in km/sec
C	backang - logical flag: Compute return leg angles?
C  Output
C	ttltim	- d.p. corrected round-trip travel time in nsec
C	lhatrn  - telescope xmit local hour angle and declination 
C		  (d.p. m and degrees)
C	azaltrn - telescope xmit az and el (d.p. m and degrees)
C	ratrn	- telescope xmit right ascension (d.p. hours)
C	lharcv  - telescope xcv local hour angle and declination 
C		  (d.p. m and degrees)
C	azalrcv - telescope xcv az and el (d.p. m and degrees)
C	rarcv	- telescope xcv right ascension (d.p. hours)
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C	12/06/05 - Eliminated agast. Removed bodftr from calls to point. rlr.
C-----------------------------------------------------------------------
  
      INCLUDE  'intrng.inc'
  
      INCLUDE  'obsinf.inc'
  
      INCLUDE  'bounce.inc'
  
      INCLUDE  'dirctn.inc'

      INCLUDE 'feature.inc'

      COMMON /deb/ debug
      LOGICAL debug
  
      DOUBLE PRECISION aberout(3), aberback(3), rotang(3)
      DOUBLE PRECISION lhatrn(6),azaltrn(6),ratrn,
     &                 lharcv(6),azalrcv(6),rarcv
      DOUBLE PRECISION nrfrvec(6), naberback(3)
      LOGICAL backang
!-----------------------------------------------------------------
!  Compute out-bound and in-bound distances and corrections
      CALL calcrng (tjdint, sutct, bodftr, backang, vlight, 
     & dtadt, dist12, dist23, corr12, corr23, aberout, aberback,
     x rotang)
  
!  Compute two way light travel time and apply necessary corrections
      if (debug) 
     &  WRITE (*,1110) dist12,dist23,corr12, corr23,dtadt-1.0d0,vlight
 1110   FORMAT ("to littim: ",f18.10,f18.10,f9.2,f9.2,f12.10,f11.4)
      CALL littim (dist12, dist23, vlight, corr12, corr23, dtadt,
     &ttltim)
      if (debug) WRITE (*,1111) ttltim,vlight
 1111            FORMAT ("from littim: ",f18.10,1x,f11.4)
  
!  Compute pointing angles (transmitter)
      legdir=1
      CALL point (tjdint+sutct/86400.d0,
     &  trfvec, aberout, transm, tgdlat, tlong,
     &  rotang,lhatrn, azaltrn,ratrn)

      if (debug) WRITE (*,*) "from t point",lhatrn,azaltrn
  
!  If transmitter not receiver...
      IF (backang) THEN
  
!  Compute pointing angles (receiver)
        legdir=2

C  Transponder test???
        DO i=1,3
          nrfrvec(i)= -rfrvec(i)
          naberback(i)= -aberback(i)
        ENDDO

        CALL point (tjdint+(sutct+ttltim)/86400.d0,
     &    nrfrvec, naberback, receiv, rgdlat, rlong,
     &    rotang,lharcv, azalrcv,rarcv)

        if (debug) WRITE (*,*) "from r point",lharcv,azalrcv

      ENDIF
  
   20 CONTINUE
! 
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE calcrng (tjdint, secutc, bodftr, backang, vlight,
     &dtadt, dist12, dist23, corr12, corr23, aberout, aberback,
     .rotang)

!  Compute outleg and inleg distances and corrections
C
C  Input
C	tjdint	- d.p. whole part of Julian date
C	secutc  - d.p. seconds of fractional Julian day
C	backang - logical flag: Compute return leg angles?
C	vlight	- d.p. velocity of light in km/sec
C  Output
C	dtadt	- Scale factor to convert from atomic to coordinate time
C	dist12	- out-bound telescope to target distance (d.p. sec)
C	dist23	- in-bound tar to telescope distance (d.p. sec)
C	corr12	- out-bound leg relativistic correction (d.p. nsec)
C	corr23	- in-bound leg relativistic correction (d.p. nsec)
C	aberout - out-bound aberration correction (d.p. x, y, z m)
C	aberback- in-bound aberration correction (d.p. x, y, z m)
C	rotang	- rotation angle (euler or ...) (d.p. degrees)
C	agast   - Greenwich apparent sidereal time (d.p. hours)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C	12/06/05 - Removed agast from call. Not used. rlr.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
  
      INCLUDE  'bounce.inc'
  
      INCLUDE  'intrng.inc'
  
      INCLUDE  'dirctn.inc'

      INCLUDE  '../include/ephihdr.inc'

      INCLUDE  '../include/ephi.inc'

      COMMON /deb/ debug
      LOGICAL debug
  
      DIMENSION bodftr(6), aberout(3), aberback(3)
      DOUBLE PRECISION agast, rotang(3)
      integer arot_type
      LOGICAL backang
  
C  Get the scale conversion between atomic and coordinate time
      tjdutc=tjdint+secutc/86400.d0
      CALL clkcor (tjdutc, transm, dtadt)
      if (debug) write(*,*) "from clkcor, dtadt=",dtadt
  
C  Convert telescope coordinates from sperical to rectangular
C  MOVE TO INITIALIZATION...
      CALL cnvcoor (1.d0, transm, bodtrn)
      DO 10 i=1,6
   10   bodrcv(i)=bodtrn(i)
C     If transmit and receive telescopes are not the same...
      IF (backang) CALL cnvcoor (1.d0, receiv, bodrcv)

C  Get the current earth orientation parameters, if present
C  NOTE: THIS IS NOT CURRENTLY USED!!
      if (ref_frame .GT. 0 .AND. nveop .GT. 0) 
     .  CALL interp_eop(tjdint, secutc, x, y, dutc, ierr)

C  Compute the outbound and inbound leg distances and corrections
      if (debug) WRITE (*,*) "station:",transm,bodtrn
      CALL ephtel (vlight, tjdint, secutc, bodtrn, bodftr,
     &trfvec,rfrvec, dist12, dist23, corr12, corr23, aberout, aberback)
CC	write(*,*) "EPHTEL OP: ",trfvec,rfrvec, dist12, dist23, 
CC     &    corr12, corr23, aberout, aberback

C  Convert scalar and vector distances from m to km
      dist12=dist12/1.d3
      dist23=dist23/1.d3
      DO i=1,6
        trfvec(i)=trfvec(i)/1.d3
        rfrvec(i)=rfrvec(i)/1.d3
      END DO

      if (debug) 
     &  WRITE (*,*) "dist12&23 corr12&23",dist12,dist23, corr12, corr23

      END
  
C-----------------------------------------------------------------------
      SUBROUTINE clkcor (tjdutc, transm, dtadt)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
C  Compute the relativistic differential relationship, in seconds, to
C  convert an interval of proper or atomic time to an interval of
C  coordinate time (see Mulholland, 1972 PASP, 84, P.357.
C
C  This effect is about 1.3 nsec for the moon  but hundreds of nsec for 
C  Mars.
C
C  Input
C	tjdutc - Julian date
C	transm - Spherical coordinates of telescope (need geocentric lat
C		 and long.
C
C  Output
C	dtadt - scale factor: a bit more than 1
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
  
      DOUBLE PRECISION transm(3)

      COMMON /twopi/ twopi,cdr,casr,ctsr
  
!......................................
      dd=tjdutc-2415020.d0
      bd=dd*1.0d-04
      bd2=bd*bd
      bd3=bd*bd2
      ems=(358.475845d0+0.985600267d0*dd-1.12d-05*bd2-0.7d-07*bd3)*cdr
      ems=dmod(ems,twopi)
      ecs=0.01675104d0-1.1444d-05*bd-9.4d-09*bd2

!......................................
!.....OBTAIN ECCENTRIC ANOMALY USING THE ABOVE PARAMETERS (SEE BROUWER
!.....AND CLEMENCE, P. 76) (same for mit and jpl ephemerides)
      u=ems+ecs*dsin(ems)
      ut=dmod(tjdutc+0.5d0,1.0d0)
      ut=ut*twopi
      elong=transm(2)*cdr
      phi=transm(3)*cdr
      cosu=dcos(u)
      dtadt=1.0d0-3.302d-10*cosu/(1.0d0-ecs*cosu)-1.476d-10*dcos(phi)*
     &dcos(ut+elong)

CC      write(*,1111) dtadt
 1111 format("dtadt= ",f15.13)
CC    write(*,*) "clk corr:",twopi,ems,ecs,u,ut,elong,phi,
CC     x    tjdutc,transm(1),transm(2),transm(3),dtadt-1.d0

      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE cnvcoor (scale, sphpos, recpos)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
C  Convert coordinates from spherical to rectangular, using input
C  scale factor.

C  Input
C	scale  - scale factor. Aukm will convert from km to au; 1 will
C		 leave km in km.
C	sphpos - body centered distance in km, east longitude and
C		 latitude both in degrees
C  Output
C	recpos - rectangular body fixed body centered coordinates
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      DOUBLE PRECISION sphpos(3),tmppos(6),recpos(6)

      COMMON /twopi/ twopi,cdr,casr,ctsr

      tmppos(1)=sphpos(1)/scale
      tmppos(2)=sphpos(2)*cdr
      tmppos(3)=sphpos(3)*cdr
      DO 10 i=4,6
        tmppos(i)=0.d0
   10 CONTINUE
      CALL rectng (tmppos, recpos)

      RETURN
      END
  
C-----------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION dlsid(dj,long)
C
C    Calculates local apparent sidereal time in terms of ut1 (i.e.
C    last1 in mueller's notation) at a place whose longitude is given
C    as the second argument. See pp. 50-53 of Explanatory Supplement to
C    the Astronomical Almanac (1992 ed.).
C    Note: this appears to be a low precision conversion... rlr.
C
C  Inputs
C      dj      - Julian date
C      long    - Longitude of site in radians
C  output
C      dlsid   - Local Mean Sidereal Time in radians
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /twopi/ twopi,cdr,casr,ctsr
 
      DOUBLE PRECISION LONG
 
C       Calculate the number of Julian centuries between the input date
C       and 1900.0.
      TJ  = (DJ-2451545.0D0)/36525.0D0
      TJ2 = TJ*TJ
      TJ3 = TJ2*TJ

      rdpts= cdr/3600.d0*15.d0
      GSTC = ((6.0D0*60.0D0+41.0D0)*60.0D0+50.54841D0)*ctsr
      GST1 = 8640184.812866D0*ctsr
      GST2 = 0.093104D0*ctsr
      GST3 = -6.2D-06*ctsr
      DDST  = DMOD(DJ+0.5d0,1.d0)*TWOPI +
     *        LONG*cdr+GSTC+GST1*TJ+GST2*TJ2+GST3*TJ3
      DLSID = DMOD(DDST,TWOPI)
      IF(DLSID.LT.0.0D0) DLSID=DLSID+TWOPI

      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE ephtel (vlight, jdint, secutc, bodtel, bodftr, 
     &  trfvec, rfrvec, dist12, dist23, corr12, corr23, 
     &  aberout, aberback)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
C 
C  compute the telescope to target vector
C
C  For the out-bound leg, iterate the light travel time
C  from the telescope to the target. 
C    - Interpolate pvac at fire time (jdint,secutc)
C    - compute x,y,z telescope to target distance (stvec) from
C	geocenter-to-target (outvec)- telescope-to-geocenter (bodtel).
C    - compute the scalar distances corresponding to outvec and stvec.
C    - The difference between these scalar distance is then the light-time
C	delay because of the geocenter-to-telescope vector. So, backup
C	the ephemeris interpolation time at the earth's center by this
C	amount, so that the firing time is referred to the time at the
C	telescope. (In other words, we are firing from the telescope, 
C	not the earth's center, and that requires a change to the 
C	interpolation time.)
C    - Go back and interpolate with the new time.
C    - Continue until the change in time is "small".
C  For the in-bound leg, iterate the light travel time
C  from the telescope to the target. 
C    - Interpolate pvac at fire time (jdint,secutc+outleg length)
C    - compute x,y,z telescope to target distance (stvec) from
C	geocenter-to-target (outvec)- telescope-to-geocenter (bodtel).
C    - We know the interpolation time is the bounce time, which is already 
C	known from the outleg computation, so we are done - no iteration is
C	needed.
C  Save the corrections and return.
C
C  Input
C	vlight	- d.p. velocity of light in km/sec
C	jdint	- d.p. whole part of Julian date
C	secutc  - d.p. seconds of fractional Julian day
C	bodtel  - d.p. 6 element array containing telescope x, y, z 
C		  coordinates and velocities (w.r.t. geocenter)
C	bodftr  - d.p. 6 element array containing lunar target x, y, z 
C  Output   - d.p. 6 body-fixed state vector, telescope to target
C	trfvec  - d.p. 6 body-fixed state vector, telescope to target (out)
C	rfrvec  - d.p. 6 body-fixed state vector, target to telescope (back)
C	dist12	- outbound one way light travel time in seconds
C	dist23	- inbound one way light travel time in seconds
C	corr12	- outbound one way relativistic correction in nsec
C	corr23	- inbound  way relativistic correction in nsec
C	aberout - d.p. 3 element array containing aberration corrections
C		  in meters. (out)
C	aberback- d.p. 3 element array containing aberration corrections
C		  in meters. (back)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------

      DIMENSION bodtel(6), bodftr(6), celtel(6), ephmtel(6)
      DIMENSION stvec(6)
  
      DOUBLE PRECISION jdutc,jdut1,jdint
      DOUBLE PRECISION olddt, corrdum
      DOUBLE PRECISION outvec(6),backvec(6)
      DOUBLE PRECISION trfvec(6),rfrvec(6)
      DOUBLE PRECISION aberout(3),aberback(3),aberdum(3)
      DOUBLE PRECISION offsetout(3),offsetback(3)
      DOUBLE PRECISION telpos(3), agast(2), rotang(3)
      DOUBLE PRECISION longm(6,3),latm(6,3), gastm(6,3)
      DOUBLE PRECISION libtmp(3,3), celftr(6)
      DOUBLE PRECISION precm(6,3), ephftr(6), ephftrr(6), JDepoch
      INTEGER arot_type
      EXTERNAL matmul
  
      COMMON /twopi/ twopi,cdr,casr,ctsr
  
      INCLUDE  'dirctn.inc'
  
      INCLUDE  '../include/ephihdr.inc'

      INCLUDE  '../include/ephi.inc'

      COMMON /deb/ debug
      LOGICAL debug

      DATA JDepoch /2451545.D0/
  
CC	debug= .true.
C  Apply outbound leg light travel time as needed...
      dt= 0.d0 

      if (debug) WRITE (*,*) "ro",vlight

   10 seci=secutc- dt
      if (debug) WRITE (*,*)"seceph:",seceph,"seci:",seci,"olddt:",olddt
  
CC	write(*,*) "olddt, dt:",olddt, dt
      CALL interp_pvac (jdint, seci, outvec, backvec, 
     &            aberout, aberdum, corr12, corrdum, 1, ierr)
      if (ierr .ne. 0) stop
  
C Outbound
        CALL matsub (outvec, bodtel, trfvec, 6)
        rnger=dsqrt(outvec(1)*outvec(1)+outvec(2)*outvec(2)+outvec(3)*
     &            outvec(3))
        dist12=
     &    dsqrt(trfvec(1)*trfvec(1)+trfvec(2)*trfvec(2)+
     &          trfvec(3)*trfvec(3))
        dt=(rnger-dist12)/vlight/1.d3

        if (debug) 
     &    WRITE (*,*) "et1",seceph,(outvec(k),k=1,3),(backvec(k),k=1,3),
     &      (trfvec(k),k=1,3),rnger,dist12,dt

! want a variable tolerance? Then FIXME
        IF (dabs(dt-olddt) .ge. 1.d-10) THEN

!    Back up the interpolation time at earth's center so that fire time for
!    the station is the same as the input time to this routine,
          olddt= dt
          GO TO 10
        ENDIF

C       Get the current offsets, if present. USE THEM: FIXME
        if (nvoff .GT. 0) then
          CALL interp_off(jdint, seci,
     x             offsetout,offsetback, ierr)
          if (ierr .ne. 0) stop
        endif

C       Get the current rotation angles, if present
C	NOTE: For the moon, this give angles adequate for pointing
C	but NOT for ranging, due to ~100 error from lack of nutations.
        if (nvrot .GT. 0) then
          CALL interp_rot(jdint,seci+dist12/vlight/1.d3,
     .               arot_type, agast(1), rotang, ierr)
          if (ierr .ne. 0) stop
          agast(1)= agast(1)*15.d0*cdr
          agast(2)= 0.d0
          if (arot_type .eq. 1) then
C           Create the libration matrix from euler angles
            call librate (rotang,libtmp)
C           Apply libration to lunar target
            call matmul(libtmp,bodftr,celftr,6,3,1)
            call precess (JDepoch,
     &                    jdint+(seci+dist12/vlight/1.d3)/86400.d0,
     &                    precm)
            call matmul(precm,celftr,ephftr,6,3,1)
C           Rotate from inertial to (earth) body fixed
            call rotmat(-3,agast,gastm)
            CALL matmul(gastm,ephftr,ephftrr,6,3,1);
            CALL matadd (trfvec, ephftrr, trfvec, 6)

          endif
C  recompute range
CC	write(*,*) "old dist12, old dt:",dist12,olddt
          olddist12= dist12
          dist12= dsqrt(trfvec(1)*trfvec(1)+trfvec(2)*trfvec(2)+
     &              trfvec(3)*trfvec(3))
          ddist12= (dist12- olddist12)/vlight/1.d3

C  Change the interpolation time by the difference in ranges.
          olddt= olddt+ ddist12
CC	write(*,*) "dist12, ddist12, old dt:",dist12,ddist12,olddt
CC	  write(*,*) "New range: ",dist12
        endif


C Inbound
C   TEST Sync Transponder Delay
      seci=secutc- olddt
      if (xpond_xmit_delay .ne. 0) seci= seci+ xpond_xmit_delay/1.d6

      CALL interp_pvac (jdint, seci, outvec, backvec, 
     &                    aberdum, aberback, corrdum, corr23, 1, ierr)
        if (ierr .ne. 0) stop

        CALL matadd (backvec, bodtel, rfrvec, 6)
        if (arot_type .eq. 1) then
            CALL matsub (rfrvec, ephftrr, rfrvec, 6)
        endif
        dist23=
     &    dsqrt(rfrvec(1)*rfrvec(1)+rfrvec(2)*rfrvec(2)+
     &          rfrvec(3)*rfrvec(3))

        if (debug) 
     &    WRITE (*,*) "et2",seceph,(outvec(k),k=1,3),(backvec(k),k=1,3),
     &      rnger,dist23,dt
  
      if (debug) 
     &  WRITE (*,*) "et3",seceph,(outvec(k),k=1,3),(backvec(k),k=1,3),
     &    (rfrvec(k),k=1,3)

      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE jdtgr(jdint,jdf,year,month,day,hour,minute,second)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

C  Calculate the gregorian date from julian date.
C
C  Input
C	jdint - D.P. Whole Julian days
C	jdf   - D.P. Fractional Julian day
C 
C  Output
C	year   - Integer year
C	month  - Integer month
C	day    - Integer day
C	hour   - Integer hour
C	minute - Integer minute
C	second - D.P. second
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------

      DOUBLE PRECISION JDINT,JDF,TDAY,JDA1900,JDFC
      INTEGER YEAR,MONTH,DAY,HOUR,MINUTE
C
      JDFC = JDF + (JDINT-DINT(JDINT)) + 0.5D0
      JDA1900 = DINT(JDINT) - 2415079.0D0
2      IF (JDFC.LT.1.D0) GO TO 5
        JDFC  = JDFC - 1.D0
        JDA1900= JDA1900 + 1.D0
        GO TO 2

 5    HOUR   = IDINT(JDFC*24.D0)
      MINUTE = IDINT((JDFC - HOUR/24.D0)*1440.D0)
      SECOND = (JDFC - HOUR/24.D0 - MINUTE/1440.D0)*86400.D0
      YEAR   = IDINT(((4.D0*JDA1900)-1)/1461.D0)
 
      TDAY   = DINT(((4.D0*JDA1900)+3.D0- (YEAR*1461.D0))/4.D0)
      MONTH  = IDINT(((5.D0*TDAY)-3.D0)/153.D0)
      DAY    = IDINT(((5.D0*TDAY)+2.D0 - (153.D0*MONTH))/5.D0)
 
      IF(MONTH.GE.10) THEN
         MONTH = MONTH - 9
         YEAR  = YEAR + 1
      ELSE
         MONTH = MONTH + 3
      ENDIF
C
         END
  
C-----------------------------------------------------------------------
      SUBROUTINE latlonm (glong, gedlat, latm, longm)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
C  Compute rotation matrices for geodetic colatitude and longitude
C
C  - Compute station geodetic colatitude in radians
C  - For the augmented rotation matrix, set the rate of the colatitude
C    to 0.0
C  - Construct augmented rotation matrix ( [g] for geodetic colatitude,
C    positive rotation through the colatitude about the 2 axis,
C    rotation flag to +2)
C  - For the augmented rotation matrix, set the rate of the longitude
C    to 0.0
C  - Construct augmented rotation matrix ( [l] for longitude, positive
C    rotation through the longitude about the 3 axis, rotation flag
C    to +3)
C
C  Input
C	glong  - Telescope longitude
C	gedlat - Telescope geodetic latitude
C  Ouput
C	latm   - colatitude rotation matrix
C	longm  - longitude rotation matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      DOUBLE PRECISION glong, gedlat
      DOUBLE PRECISION colat(2),long(2),latm(6,3),longm(6,3)
      COMMON /twopi/ twopi,cdr,casr,ctsr

      colat(1)=twopi/4.d0-gedlat*cdr
      colat(2)=0.d0
      CALL rotmat (+2, colat, latm)
      long(1)=glong*cdr
      long(2)=0.d0
      CALL rotmat (+3, long, longm)

      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE librate(libr,rm)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C
C  Compute the lunar libration rotation matrix rm using the information
C  contained in the libration (euler) angles from the jpl ephemeris.
C
C  Note: the derivative positions are not populated.
C
C  Input
C	libr - d.p. euler libration angles (radians??)
C
C  Output
C	rm   - d.p. rotation matrix
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      DOUBLE PRECISION libr(3), rm(6,3)

      COMMON /deb/ debug
      LOGICAL debug

      DO 100 I=1,6
      DO 100 J=1,3
  100 RM(I,J)=0.0D0

      c1=dcos(libr(1))
      c2=dcos(libr(2))
      c3=dcos(libr(3))
      s1=dsin(libr(1))
      s2=dsin(libr(2))
      s3=dsin(libr(3))
      rm(1,1) = c3* c1 - c2* s1* s3
      rm(2,1) = c3* s1 + c2* c1* s3
      rm(3,1) = s3* s2
      rm(1,2) =-s3* c1 - c2* s1* c3
      rm(2,2) =-s3* s1 + c2* c1* c3
      rm(3,2) = c3* s2
      rm(1,3) = s2* s1
      rm(2,3) =-s2* c1
      rm(3,3) = c2

      IF (debug) WRITE(*,1001) rm
 1001 FORMAT(1X,"LIBR: (jpl) ",/,1X,6(3E20.13/))

      return
      end
  
C-----------------------------------------------------------------------
      SUBROUTINE littim (dist12, dist23, vlight, corr12, corr23,
     &dtadt, ttltim)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)

C  Compute two way light travel time and apply necessary corrections
C
C  - Compute round trip range distance by adding the out bounce leg
C    distance to the bounce return leg distance.
C  - Convert round trip range distance in km to light travel time in seco
C  - Add the out bounce and bounce return corrections for geodetic 
C    curvature and convert to seconds. 
C  - Add the summed geodesic curvature corrections to the round trip
C    light travel time.
C  - Add the approximate e-m geodesic curvature correction to the round
C    trip light travel time.
C  - Multiply the corrected light travel time by the relativistic
C    clock correction to convert from coordinate to proper time.
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
  
      range=dist12+dist23
      ttltim=range/vlight+(corr12-corr23)*1.d-9+0.27d-9
!                                                 .27=APPROX E-M REL COR

CC      write(*,1000) dist12/vlight,dist23/vlight,range/vlight,
CC     &              corr12,corr23,ttltim
CC 1000	format("TTLTIM: ",6(1x,f20.10))

      ttltim=ttltim*dtadt

      END

C-----------------------------------------------------------------------
      SUBROUTINE matadd (a, b, c, nrow)
C
C  Add vectors a and b with results in vector c.
C
C  Input: 
C     a = 6 element DP vector
C     b = 6 element DP vector
C     nrow = number of rows of matrices a, b and c
C
C  Output:
C     b = 6 element DP vector
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DOUBLE PRECISION a(6),b(6),c(6)
      DO 10 i=1,nrow
        c(i)=a(i)+b(i)
   10 CONTINUE
      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE matmpy (a, b, c, nrow, ncol)
C
C  Premultiply matrix b by matrix a with results in matrix c
C
C  Input:
C     a = input matrix
C     b = input matrix
C     nrow = number of rows of matrix A
C     ncol = number of columns of matrix b
C  Output:
C     c = output matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DIMENSION a(3,3), b(3,3), c(3,3)
      DO 30 i=1,nrow
        DO 20 j=1,ncol
          sum=0.0
          DO 10 n=1,nrow
            sum=sum+a(i,n)*b(n,j)
   10     CONTINUE
          c(i,j)=sum
   20   CONTINUE
   30 CONTINUE
      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE matmul (a, b, c, nrows, ncola, ncolb)
C
C  Multiply matrices a and b with the result being in c.
C
C  Input: 
C    a - nrow x ncola DP matrix
C    b - nrow x ncola DP matrix
C    nrows - number of rows in matrices
C    ncola - number of columns in input matrices
C    ncolb - number of columns in output matrix
C   
C  Output:
C    c - nrow x ncolb DP matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DOUBLE PRECISION a(6,3),b(6,3),c(6,3)
      DOUBLE PRECISION p(3,3),v(3,3)
 
      nrw2=nrows/2
      imax=nrw2
      jmax=ncolb
      kmax=ncola
 
      DO 10 j=1,jmax
        DO 10 i=1,imax
        p(i,j)=0.0
        v(i,j)=0.0
        DO 10 k=1,kmax
        m1=i
        m2=k
        m3=k
        m4=j
        p(i,j)=p(i,j)+a(m1,m2)*b(m3,m4)
        v(i,j)=v(i,j)+a(m1,m2)*b(m3+3,m4)+a(m1+3,m2)*b(m3,m4)
   10 CONTINUE
      DO 20 j=1,jmax
        DO 20 i=1,imax
        c(i,j)=p(i,j)
   20   c(i+3,j)=v(i,j)
      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE matsub (a, b, c, nrow)
C
C  Subtract vector b from vector a with result in vector c.
C
C  Input: 
C     a = 6 element DP vector
C     b = 6 element DP vector
C     nrow = number of rows of matrices a, b and c
C
C  Output:
C     b = 6 element DP vector
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DOUBLE PRECISION a(6),b(6),c(6)

      DO 10 i=1,nrow
        c(i)=a(i)-b(i)
   10 CONTINUE
      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE mattrns (a, nrow, ncol)
C
C  Compute transpose of an augmented matrix
C
C  Input:
C    a - matrix to transpose
C    nrow, ncol - rows and columns in matrix a
C
C  Output:
C    a - result of transposition
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DOUBLE PRECISION a(6,3),b(3,3)

!.....First, positions
      nrw2=nrow/2
      nrw2p1=nrw2+1
      DO 20 i=1,nrw2
        DO 10 j=1,ncol
          b(j,i)=a(i,j)
   10   CONTINUE
   20 CONTINUE
      DO 40 i=1,nrw2
        DO 30 j=1,ncol
          a(i,j)=b(i,j)
   30   CONTINUE
   40 CONTINUE
!.....Then velocities
      DO 60 i=1,nrw2
        DO 50 j=1,ncol
          b(j,i)=a(i+nrw2,j)
   50   CONTINUE
   60 CONTINUE
      DO 80 i=1,nrw2
        DO 70 j=1,ncol
          a(i+nrw2,j)=b(i,j)
   70   CONTINUE
   80 CONTINUE
      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE point (jd,tlrfvec, aberat, telpos, gedlat, long, 
     &rotang, lhasph, azalsph, ra)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)

C  Compute pointing angles
C
C  Input
C    jd   - Julian date
C    tlrfvec - Telescope to reflector (or center of moon) vector
C    aberat  - Aberration corrections in geocentric x, y, z (d.p. m)
C    telpos  - Coordinates of telescope (d.p. radius, geoc. lat, long - m, deg)
C    gedlat  - Geodetic latitude of the telescope (d.p. degrees)
C    long    - Longitude of telescope (d.p. degrees)
C    agast   - Greenwich Apparent Sidereal Time (d.p. hours)
C    rotang  - Rotation angles, def'n depending on rot_type. (d.p. degrees)
C  Output
C    lhasph  - telescope local hour angle and declination (d.p. m and degrees)
C    azalsph - telescope az and el (d.p. m and degrees)
C    ra	     - telescope right ascension (d.p. hours)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C	12/06/05 - remove agast and bodftr from call. Not used. rlr.
C-----------------------------------------------------------------------
  
      INCLUDE  'bounce.inc'
  
      INCLUDE  'dirctn.inc'
  
      INCLUDE  '../include/ephihdr.inc'

      INCLUDE  '../include/ephi.inc'

      COMMON /twopi/ twopi,cdr,casr,ctsr

      COMMON /deb/ debug
      LOGICAL debug
  
      DOUBLE PRECISION jd, gdlat, long
      DOUBLE PRECISION telpos(3), agast, rotang(3)
      DOUBLE PRECISION tlrfvec(6),aberat(3),bodtel(6),lhatel(6)
      DOUBLE PRECISION lhasph(6),azaltel(6),azalsph(6),libtmp(3,3)
      DOUBLE PRECISION longm(6,3),latm(6,3), gastm(6,3)
      DOUBLE PRECISION ra, telftr(3)
      EXTERNAL matmul

C Apply "Stellar" aberration
      DO i=1,3
        if (legdir .eq. 1) then
           bodtel(i)=tlrfvec(i)+aberat(i)/1.d3
        else
           bodtel(i)=tlrfvec(i)-aberat(i)/1.d3
        endif
      ENDDO

      if (debug) WRITE (*,*) "abcorr",aberat
      if (debug) WRITE (*,*) "abcorr",aberat,"tlrfvec",tlrfvec,
     x  "coords",telpos(2), gedlat
 
C True body Fixed to topocentric ha/dec 
      CALL latlonm (telpos(2), gedlat, latm, longm)
      if (debug) write(*,*) "Telpos: ",telpos," gedlat: ",gedlat
      CALL mattrns (longm, 6, 3)
      CALL matmul (longm, bodtel, lhatel, 6, 3, 1)
      if (debug) write(*,*) "bodtel: ",bodtel," lhatel: ",lhatel
      CALL sphere (lhatel, lhasph)

C topocentric ha/dec to topocentric az/el
      CALL mattrns (latm, 6, 3)
      CALL matmul (latm, lhatel, azaltel, 6, 3, 1)
      CALL sphere (azaltel, azalsph)
  
C Clean up angles
      azalsph(2)=180.d0-azalsph(2)/cdr
      IF (azalsph(2).lt.0.d0) azalsph(2)=azalsph(2)+360.d0
      azalsph(3)=azalsph(3)/cdr
C     The program displays RA, but it isn't important or worth the
C     effort to get it right. Maybe it should be dropped. rlr. 05/2005
CC      ra= (dlsid(jd,long)+ lhasph(2))/cdr/15.d0+ 12.d0
      ra= (dlsid(jd,long)+ lhasph(2))/cdr/15.d0
      IF (ra .gt. 24.d0) ra= ra- 24.d0
      lhasph(2)=-lhasph(2)/cdr/15.d0
      lhasph(3)=lhasph(3)/cdr

      if (debug) write(*,*) "jd,jdf,long,lst,lha,ra=",
     x		jd,jd- dint(jd),long,dlsid(jd,long)/cdr/15.d0,
     x		lhasph(2)/cdr/15.d0,ra

      RETURN
      END
  
C-----------------------------------------------------------------------
      SUBROUTINE precess(t1,t2,precm)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C Compute augmented rotation matrix for precession
C
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------

      DOUBLE PRECISION precm(6,3)
      DOUBLE PRECISION mtrx1(6,3),mtrx2(6,3),mtrx3(6,3),temp1(6,3)
      DOUBLE PRECISION zeta0(2),theta(2),z(2)
      DOUBLE PRECISION jd2000
      ExTERNAL matmul

      COMMON/twopi/twopi,cdr,casr,ctsr
      DATA jd2000/2451545.D0/
      DATA tcent/36525.D0/

      t0 = (t1 - jd2000)/tcent
      t = (t2 - t1)/tcent
      a1 = (-0.000139d0*t0+1.39656d0)*t0+2306.2181d0
      a2 = 0.30188d0-0.000345d0*t0
      a3 = 0.017998d0
      b1 = a1
      b2 = 1.09468d0+0.000066d0*t0
      b3 = 0.018203d0
      c1 = (-0.000217d0*t0-0.85330d0)*t0+2004.3109d0
      c2 = -0.42665d0-0.000217d0*t0
      c3 = -0.041833d0
      zeta0(1) = (((a3*t + a2)*t + a1)*t)*casr
      z(1)     = (((b3*t + b2)*t + b1)*t)*casr
      theta(1) = (((c3*t + c2)*t + c1)*t)*casr
      zeta0(2) = ((3*a3*t + 2*a2)*t + a1)/tcent*casr
      z(2)     = ((3*b3*t + 2*b2)*t + b1)/tcent*casr
      theta(2) = ((3*c3*t + 2*c3)*t + c1)/tcent*casr
      CALL rotmat(+3,zeta0,mtrx1)
      CALL rotmat(-2,theta,mtrx2)
      CALL rotmat(+3,z,    mtrx3)
      CALL matmul(mtrx2,mtrx1,temp1,6,3,3)
      CALL matmul(mtrx3,temp1,precm,6,3,3)

      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE rectng (sphere, xyz)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)

C  Compute rectangular coordinates from spherical coordinates
C
C  Input
C	sphere - d.p. 6 element vector (radius, long, geoc lat, and 
C		 velocities. Angles in radians)
C  Output
C	xyz    - d.p. 6 element vector (x, y, z, xdot, ydot, zdot)
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      DOUBLE PRECISION sphere(6),xyz(6)

      sinlon=dsin(sphere(2))
      coslon=dcos(sphere(2))
      sinlat=dsin(sphere(3))
      coslat=dcos(sphere(3))
      cc=coslat*coslon
      cs=coslat*sinlon
      sc=sinlat*coslon
      ss=sinlat*sinlon
      xyz(1)=sphere(1)*cc
      xyz(2)=sphere(1)*cs
      xyz(3)=sphere(1)*sinlat
      xyz(4)=sphere(4)*cc-sphere(1)*(sphere(5)*cs+sphere(6)*sc)
      xyz(5)=sphere(4)*cs+sphere(1)*(sphere(5)*cc-sphere(6)*ss)
      xyz(6)=sphere(4)*sinlat+sphere(1)*sphere(6)*coslat

      END

C-----------------------------------------------------------------------
      SUBROUTINE rotmat (kaxis, a, r)
C
C  Construct augmented rotation matrix
C  Input:
C     kaxis   - Rotation axis: -1, -2, -3, or 1, 2, or 3
C	        Negative implies negative rotation (i.e., negate a)
C     a(2)    - angle and angle rate 
C
C  Output:
C     r (6,3) - rotation matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DIMENSION r(6,3), a(2)
  
      DO 10 j=1,6
        DO 10 i=1,3
   10   r(j,i)=0.0d0
  
      k=iabs(kaxis)
      ione=k/kaxis
      a1=ione*a(1)
      adot=ione*a(2)
  
      sina=dsin(a1)
      cosa=dcos(a1)
      adsina=adot*sina
      adcosa=adot*cosa
  
      kp1=mod(k,3)+1
      kp2=mod(kp1,3)+1
      r(k,k)=1.0d0
      r(kp1,kp1)=cosa
      r(kp1,kp2)=-sina
      r(kp2,kp1)=sina
      r(kp2,kp2)=cosa
      r(kp1+3,kp1)=-adsina
      r(kp1+3,kp2)=-adcosa
      r(kp2+3,kp1)=adcosa
      r(kp2+3,kp2)=-adsina

      END
  
C-----------------------------------------------------------------------
      SUBROUTINE sphere (xyz, spher)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
C  Compute spherical coordinates from rectangular coordinates

C	rho*rho = x*x + y*y
C	rho = sqrt(rho*rho)
C	r = sqrt(rho*rho + z*z)
C	long = atan(y/x)
C	lat = atan(z/rho)
C
C  Input
C	xyz - rectangular coordinates (x,y,z,dx,dy,dz)
C  Output
C	spher - Spherical coordinates (radius, lat, long, 
C		dradius, dlat, dlong)
C	        Velocities are set to 0.
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------
      DIMENSION xyz(6), spher(6)
      rhosq=xyz(1)*xyz(1)+xyz(2)*xyz(2)
      rho=dsqrt(rhosq)
      spher(1)=dsqrt(rhosq+xyz(3)*xyz(3))
      spher(2)=datan2(xyz(2),xyz(1))
      spher(3)=datan2(xyz(3),rho)
      DO 10 i=4,6
        spher(i)=0.d0
   10 CONTINUE

      RETURN
      END
